package com.smcic.api.epg.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.smcic.api.epg.dto.DisableDTO;
import com.smcic.api.epg.entity.Channel;
import com.smcic.api.epg.entity.EpgDisableConfig;
import com.smcic.api.epg.entity.Program;
import com.smcic.api.epg.mapper.EpgDisableConfigMapper;
import com.smcic.api.epg.service.IEpgDisableConfigService;
import com.smcic.core.advice.APIException;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;

@Service
public class EpgDisableConfigService extends ServiceImpl<EpgDisableConfigMapper, EpgDisableConfig> implements IEpgDisableConfigService {

    @Resource
    private ChannelService channelService;

    @Resource
    private ProgramService programService;

    public Map<String, EpgDisableConfig> cfMap(Integer channelId, LocalDateTime start, LocalDateTime end){
        Map<String, EpgDisableConfig> map = new HashMap<>();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
        lambdaQuery().eq(EpgDisableConfig::getChannelId, channelId).ge(EpgDisableConfig::getStartTime, start)
                .le(EpgDisableConfig::getStartTime, end).eq(EpgDisableConfig::getStatus, 1).list()
                .forEach(sspDisableConfig -> map.put(sspDisableConfig.getProgram() + "_" + sspDisableConfig.getStartTime().format(formatter), sspDisableConfig));
        return map;
    }

    public void disable(DisableDTO disableDTO){
        Channel channel = channelService.getBaseMapper().channelById(disableDTO.getChannelId());
        if(null == channel){
            throw new APIException("不存在的频率频道");
        }
        Program program = programService.getBaseMapper().byStartAndChannel(disableDTO.getChannelId(), disableDTO.getStartTime(), disableDTO.getTitle());

        if(null == program){
            throw new APIException("不存在的节目");
        }
        EpgDisableConfig epgDisableConfig;

        epgDisableConfig = lambdaQuery().eq(EpgDisableConfig::getChannelId, disableDTO.getChannelId())
                .eq(EpgDisableConfig::getStartTime, disableDTO.getStartTime())
                .eq(EpgDisableConfig::getProgram, disableDTO.getTitle()).one();

        if(epgDisableConfig == null){
            epgDisableConfig = new EpgDisableConfig();
            epgDisableConfig.setChannelId(disableDTO.getChannelId());
            epgDisableConfig.setChannelName(channel.getTitle());
            epgDisableConfig.setStartTime(disableDTO.getStartTime());
            epgDisableConfig.setProgram(disableDTO.getTitle());
            epgDisableConfig.setStatus(disableDTO.getStatus());
            save(epgDisableConfig);
        }else {
            epgDisableConfig.setStatus(disableDTO.getStatus());
            updateById(epgDisableConfig);
        }
        programService.setCache(disableDTO.getChannelId());
    }

    public void update(DisableDTO disableDTO){
        EpgDisableConfig edc = getById(disableDTO.getId());

        if(null == edc){
            throw new APIException("不存在的数据记录");
        }

        edc.setStatus(disableDTO.getStatus());

        updateById(edc);

    }
}
