package com.smcic.api.epg.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.smcic.api.epg.dto.DisableDTO;
import com.smcic.api.epg.entity.Channel;
import com.smcic.api.epg.entity.EpgDisableConfig;
import com.smcic.api.epg.entity.KeywordDisableConfig;
import com.smcic.api.epg.entity.Program;
import com.smcic.api.epg.mapper.EpgDisableConfigMapper;
import com.smcic.api.epg.mapper.KeywordDisableConfigMapper;
import com.smcic.api.epg.service.IEpgDisableConfigService;
import com.smcic.api.epg.service.IKeywordDisableConfigService;
import com.smcic.core.advice.APIException;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class KeywordDisableConfigService extends ServiceImpl<KeywordDisableConfigMapper, KeywordDisableConfig> implements IKeywordDisableConfigService {

    @Resource
    private ChannelService channelService;

    @Resource
    private ProgramService programService;

    public void disable(KeywordDisableConfig keywordDisableConfig){
        Channel channel = channelService.getBaseMapper().channelById(keywordDisableConfig.getChannelId());
        if(null == channel){
            throw new APIException("不存在的频率频道");
        }

        KeywordDisableConfig disableConfig = new KeywordDisableConfig();
        disableConfig.setChannelId(keywordDisableConfig.getChannelId());
        disableConfig.setChannelName(channel.getTitle());
        disableConfig.setStartTime(keywordDisableConfig.getStartTime());
        disableConfig.setEndTime(keywordDisableConfig.getEndTime());
        disableConfig.setProgram(keywordDisableConfig.getProgram());
        disableConfig.setStatus(1);
        save(disableConfig);
        programService.setCache(keywordDisableConfig.getChannelId());
    }

    public IPage<KeywordDisableConfig> keywordConfigList(Integer channelId, Integer page, Integer pageSize){
        LambdaQueryWrapper<KeywordDisableConfig> wrapper = Wrappers.lambdaQuery();
        wrapper = wrapper.eq(KeywordDisableConfig::getChannelId, channelId).orderByDesc(KeywordDisableConfig::getId);
        return baseMapper.selectPage(new Page<>(page, pageSize), wrapper);
    }

    public List<String> disableKeywords(Integer channelId, LocalDateTime startTime, LocalDateTime endTime){
        return lambdaQuery().eq(KeywordDisableConfig::getChannelId, channelId).eq(KeywordDisableConfig::getStatus, 1)
                .le(KeywordDisableConfig::getEndTime, startTime).ge(KeywordDisableConfig::getStartTime, endTime)
                .list().stream().map(KeywordDisableConfig::getProgram).distinct().collect(Collectors.toList());
    }

    public void update(KeywordDisableConfig keywordDisableConfig){
        KeywordDisableConfig kdc = getById(keywordDisableConfig.getId());

        if (null == kdc) {
            throw new APIException("不存在的配置");
        }

        kdc.setProgram(keywordDisableConfig.getProgram());
        kdc.setStartTime(keywordDisableConfig.getStartTime());
        kdc.setEndTime(keywordDisableConfig.getEndTime());
        kdc.setStatus(keywordDisableConfig.getStatus());
        updateById(keywordDisableConfig);
    }
}
